/******************************************************************/
/* ACM ICPC 2012-2013                                             */
/* Northeastern European Regional Contest                         */
/* St Petersburg - Barnaul - Tashkent - Tbilisi, December 2, 2012 */
/******************************************************************/
/* Problem C. Caravan Robbers                                     */
/*                                                                */
/* Original idea         Egor Kulikov                             */
/* Problem statement     Egor Kulikov                             */
/* Test set              Egor Kulikov                             */
/******************************************************************/
/* Solution                                                       */
/*                                                                */
/* Author                Egor Kulikov                             */
/******************************************************************/

import java.io.*;
import java.util.InputMismatchException;

/**
 * @author egor@egork.net
 */
public class caravan_kulikov {
	static final String ID = "caravan";
	static final int MAXN = 100000;
	static final int MINN = 1;
	static final int MAXX = 1000000;
	static final int MINX = 0;

	public static void main(String[] args) throws FileNotFoundException {
		InputReader in = new InputReader(new FileInputStream(ID + ".in"));
		PrintWriter out = new PrintWriter(ID + ".out");
		new caravan_kulikov().solve(in, out);
		out.close();
	}

	long t = System.currentTimeMillis();

	void solve(InputReader in, PrintWriter out) {
		int count = in.readInt();
		final int[] start = new int[count];
		int[] end = new int[count];
		for (int i = 0; i < count; i++) {
			start[i] = in.readInt();
			end[i] = in.readInt();
		}
		output("Reading");
		int[] order = validateTest(count, start, end);
		output("Validation");
		double left = 1;
		double right = (double)MAXX / count;
		for (int i = 0; i < 50; i++) {
			double middle = (left + right) / 2;
			boolean ok = true;
			double current = 0;
			for (int j : order) {
				current = Math.max(current, start[j]) + middle;
				if (current > end[j]) {
					ok = false;
					break;
				}
			}
			if (ok)
				left = middle;
			else
				right = middle;
		}
		output("Binary search");
		double exact = (left + right) / 2;
		int numerator = 1;
		int denominator = 1;
		double answer = 1;
		for (int i = 1; i <= count; i++) {
			int candidateNumerator = (int) Math.round(exact * i);
			double value = (double)candidateNumerator / i;
			if (Math.abs(value - exact) < Math.abs(answer - exact)) {
				numerator = candidateNumerator;
				denominator = i;
				answer = value;
			}
		}
		output("Answer");
		validate(start, end, numerator, denominator, order);
		output("Answer validation");
		int g = gcd(numerator, denominator);
		numerator /= g;
		denominator /= g;
		out.println(numerator + "/" + denominator);
	}

	public static int[] order(final int[] array) {
		return sort(createOrder(array.length), new IntComparator() {
			public int compare(int first, int second) {
				if (array[first] < array[second])
					return -1;
				if (array[first] > array[second])
					return 1;
				return 0;
			}
		});
	}

	public static int[] createOrder(int size) {
		int[] order = new int[size];
		for (int i = 0; i < size; i++)
			order[i] = i;
		return order;
	}

	public static int[] sort(int[] array, IntComparator comparator) {
		return sort(array, 0, array.length, comparator);
	}

	public static int[] sort(int[] array, int from, int to, IntComparator comparator) {
		ensureCapacityInt(to - from);
		System.arraycopy(array, from, tempInt, 0, to - from);
		sortImpl(array, from, to, tempInt, 0, to - from, comparator);
		return array;
	}

	private static int[] tempInt = new int[0];

	private static void ensureCapacityInt(int size) {
		if (tempInt.length >= size)
			return;
		size = Math.max(size, tempInt.length << 1);
		tempInt = new int[size];
	}

	private static void sortImpl(int[] array, int from, int to, int[] temp, int fromTemp, int toTemp, IntComparator comparator) {
		if (to - from <= 1)
			return;
		int middle = (to - from) >> 1;
		int tempMiddle = fromTemp + middle;
		sortImpl(temp, fromTemp, tempMiddle, array, from, from + middle, comparator);
		sortImpl(temp, tempMiddle, toTemp, array, from + middle, to, comparator);
		int index = from;
		int index1 = fromTemp;
		int index2 = tempMiddle;
		while (index1 < tempMiddle && index2 < toTemp) {
			if (comparator.compare(temp[index1], temp[index2]) <= 0)
				array[index++] = temp[index1++];
			else
				array[index++] = temp[index2++];
		}
		if (index1 != tempMiddle)
			System.arraycopy(temp, index1, array, index, tempMiddle - index1);
		if (index2 != toTemp)
			System.arraycopy(temp, index2, array, index, toTemp - index2);
	}

	private void output(String s) {
		long current = System.currentTimeMillis();
		System.err.println(s + " " + (current - t));
		t = current;
	}

	static int[] validateTest(int count, final int[] start, int[] end) {
		if (count < MINN || count > MAXN)
			throw new RuntimeException("n = " + count);
		if (start.length != count || end.length != count)
			throw new RuntimeException("length mismatch");
		for (int i = 0; i < count; i++) {
			if (start[i] < MINX || start[i] >= end[i] || end[i] > MAXX)
				throw new RuntimeException("segment " + start[i] + ", " + end[i]);
		}
		int[] order = order(start);
		for (int i = 1; i < count; i++) {
			if (start[order[i]] == start[order[i - 1]] || end[order[i]] <= end[order[i - 1]])
				throw new RuntimeException("contains " + start[order[i]] + ", " + end[order[i]] + " in " + start[order[i - 1]] + ", " + end[order[i - 1]]);
		}
		return order;
	}

	static void validate(int[] start, int[] end, int numerator, int denominator, int[] order) {
		long current = 0;
		boolean exact = false;
		for (int i : order) {
			current = Math.max(current, (long)start[i] * denominator) + numerator;
			if (current > (long)end[i] * denominator)
				throw new RuntimeException("Answer too big " + numerator + "/" + denominator);
			if (current == (long)end[i] * denominator)
				exact = true;
		}
		if (!exact)
			throw new RuntimeException("Answer too small" + numerator + "/" + denominator);
	}

	static int gcd(int a, int b) {
		if (b == 0)
			return a;
		return gcd(b, a % b);
	}

	static class InputReader {
		private InputStream stream;
		private byte[] buf = new byte[1024];
		private int curChar;
		private int numChars;

		public InputReader(InputStream stream) {
			this.stream = stream;
		}

		public int read() {
			if (numChars == -1)
				throw new InputMismatchException();
			if (curChar >= numChars) {
				curChar = 0;
				try {
					numChars = stream.read(buf);
				} catch (IOException e) {
					throw new InputMismatchException();
				}
				if (numChars <= 0)
					return -1;
			}
			return buf[curChar++];
		}

		public int readInt() {
			int c = read();
			while (isSpaceChar(c))
				c = read();
			int sgn = 1;
			if (c == '-') {
				sgn = -1;
				c = read();
			}
			int res = 0;
			do {
				if (c < '0' || c > '9')
					throw new InputMismatchException();
				res *= 10;
				res += c - '0';
				c = read();
			} while (!isSpaceChar(c));
			return res * sgn;
		}

		public String readString() {
			int c = read();
			while (isSpaceChar(c))
				c = read();
			StringBuffer res = new StringBuffer();
			do {
				res.appendCodePoint(c);
				c = read();
			} while (!isSpaceChar(c));
			return res.toString();
		}

		public boolean isSpaceChar(int c) {
			return c == ' ' || c == '\n' || c == '\r' || c == '\t' || c == -1;
		}

		private String readLine0() {
			StringBuffer buf = new StringBuffer();
			int c = read();
			while (c != '\n' && c != -1) {
				if (c != '\r')
					buf.appendCodePoint(c);
				c = read();
			}
			return buf.toString();
		}

		public String readLine() {
			String s = readLine0();
			while (s.trim().length() == 0)
				s = readLine0();
			return s;
		}

		public String readLine(boolean ignoreEmptyLines) {
			if (ignoreEmptyLines)
				return readLine();
			else
				return readLine0();
		}

		public String next() {
			return readString();
		}

	}

	static interface IntComparator {
		public int compare(int first, int second);
	}

}

