/******************************************************************/
/* ACM ICPC 2012-2013                                             */
/* Northeastern European Regional Contest                         */
/* St Petersburg - Barnaul - Tashkent - Tbilisi, December 2, 2012 */
/******************************************************************/
/* Problem G. Great Deceiver                                      */
/*                                                                */
/* Original idea         Elana Kryuchkova                         */
/* Problem statement     Dmitry Gozman                            */
/* Test set              Dmitry Gozman                            */
/******************************************************************/
/* Tests                                                          */
/*                                                                */
/* Author                Dmitry Gozman                            */
/******************************************************************/

import java.io.*;
import java.util.*;

public class Tests {
	
	private static int TEST_COUNTER = 0;
	private static long MAXN = Math.round(Math.pow(10, 15));
	private static int MAXK = 1000;
	private static Random random = new Random(42);
	
	private static void test(long n, int k) throws IOException {
		File testDir = new File("tests");
		testDir.mkdir();
		
		String tn = String.format("%02d", ++TEST_COUNTER);
		assert 1 <= n && n <= MAXN : "Wrong N at test " + tn;
		assert 2 <= k && k <= MAXK : "Wrong k at test " + tn;
		PrintWriter writer = new PrintWriter(new FileWriter(new File(testDir, tn)));
		writer.println(n + " " + k);
		writer.close();
	}

	public static void main(String[] args) throws IOException {
		// Samples
		test(21, 3);
		test(21, 2);
		
		// Small tests.
		test(1, 1000);
		test(1, 17);
		test(42, 42);
		
		// Excessive testing for k = 3.
		for (int n = 1; n <= 9; n++) {
			test(n, 3);
		}
		test(9 + 3, 3);
		test(2 * 9 - 2 * 3 + 2, 3);
		test(2 * 9 + 1 * 3 + 1, 3);
		test(27 + 9 + 3 + 1, 3);
		test(1 * 81 - 1 * 27 + 1 * 9 - 1 * 3 + 1, 3);
		test(2 * 81 - 2 * 27 + 1 * 9 - 1 * 3 + 0, 3);
		
		// Numbers with zeroes on even positions.
		for (int len : new int[] {1, 3, 5, 7, 9}) {
			for (int k : new int[] {2, 6, 17, 25, 42}) {
				zeroes(len, k);
			}
		}
		
		// Max zeroes tests.
		for (int k : new int[] {2, 3, 15, 86, 124, 511, 707, 997, 1000}) {
			maxZeroes(k);
		}
		
		// Partial zeroes tests.
		for (int k : new int[] {2, 7, 8, 22, 61, 99, 333, 478, 800}) {
			partialZeroes(k);
		}
		
		// Max tests.
		for (long n = MAXN - 1; n <= MAXN; n++) {
			for (int k = MAXK - 1; k <= MAXK; k++) {
				test(n, k);
			}
		}
		
		// Random tests.
		for (int i = 0; i < 30; i++) {
			long n = random.nextLong();
			n = (n % MAXN + MAXN) % MAXN + 1;
			int k = random.nextInt(MAXK - 1) + 2;
			test(n, k);
		}
	}

	private static void partialZeroes(int k) throws IOException {
		long n = random.nextInt(k - 1) + 1;
		int num = 0;
		while (n + 1 <= MAXN / k / k) {
			int d = random.nextInt(k);
			n = n * k * k + d;
			num++;
		}
		
		int take = random.nextInt(num - 1) + 1;
		long add = 0;
		for (int i = 0; i < take; i++) {
			add = add * k * k + random.nextInt(k - 1) + 1;
		}
		
		test(n + add, k);
	}

	private static void maxZeroes(int k) throws IOException {
		long n = random.nextInt(k - 1) + 1;
		while (n <= (MAXN - (k - 1)) / k / k) {
			int d = random.nextInt(k);
			n = n * k * k + d;
		}
		test(n, k);
	}

	private static void zeroes(int len, int k) throws IOException {
		long n = 0;
		for (int i = 0; i < len; i++) {
			int d = 0;
			if (i % 2 == 0) {
				d = i == 0 ? random.nextInt(k - 1) + 1 : random.nextInt(k); 
			}
			
			n = n * k + d;
		}
		test(n, k);
	}
}

