/******************************************************************/
/* ACM ICPC 2012-2013                                             */
/* Northeastern European Regional Contest                         */
/* St Petersburg - Barnaul - Tashkent - Tbilisi, December 2, 2012 */
/******************************************************************/
/* Problem G. Great Deceiver                                      */
/*                                                                */
/* Original idea         Elana Kryuchkova                         */
/* Problem statement     Dmitry Gozman                            */
/* Test set              Dmitry Gozman                            */
/******************************************************************/
/* Solution                                                       */
/*                                                                */
/* Author                Roman Elizarov                           */
/******************************************************************/

import java.io.*;

/**
 * Solution for NEERC'2012 Problem G: Great Deceiver
 * This solution checks correctness of the input.
 *
 * @author Roman Elizarov
 */
public class great_re {

    private static final long MAX_N = 1_000_000_000_000_000L;
    private static final int MAX_K = 1000;

    private static final int MAX_ND = 64;

    public static void main(String[] args) throws Exception {
        new great_re().go();
    }

    void go() throws Exception {
        long time = System.currentTimeMillis();
        read();
        solve();
        write();
        System.out.printf("Done in %d ms%n", System.currentTimeMillis() - time);
    }

    long n;
    int k;

    void read() throws Exception {
        Scanner in = new Scanner(new File("great.in"));
        n = in.nextLong();
        k = in.nextInt();
        assert n >= 1 && n <= MAX_N;
        assert k >= 2 && k <= MAX_K;
        in.nextLine();
        in.close();
    }

    long result;

    void solve() {
        // find digits base k
        int[] d = new int[MAX_ND];
        int nd = 0;
        while (n > 0) {
            d[nd++] = (int)(n % k);
            n /= k;
        }
        // compute result
        result = 1; // just zero
        // for all number with less than nd digits
        long nn = 1;
        for (int i = 0; i < nd - 1; i++) {
            // for numbers xyyyy...y (i times) where x is non zero (from 1 to k-1)
            if (i % 2 != 0)
                continue; // can't have non zero at odd position
            result += nn * (k - 1); // inc by (k-1)*k^t, where t is the number of other even position where we can have 0-9
            nn *= k;
        }
        if ((nd - 1) % 2 != 0)
            return; // that's it. cannot have non-zero at last digit's position
        // last digit from 1 to the max one (no inclusive)
        result += nn * (d[nd - 1] - 1);
        // no last digit is max one, prev digit goes to max one, etc
        for (int i = nd - 2; i >= 0; i--) {
            if (i % 2 != 0) {
                if (d[i] > 0) { // this digit has to be zero, but prev ones take any value
                    result += nn;
                    return; // that's it. No more larger numbers
                }
                nn /= k;
                continue;
            }
            result += nn * d[i];
        }
        result++; // count N itself -- last digit takes its max value
    }

    void write() throws Exception {
        System.out.println("Result = " + result);
        PrintWriter out = new PrintWriter("great.out");
        out.println(result);
        out.close();
    }

//----------------- just for validation ------------------

    /**
     * Strict scanner to verify 100% correspondence between input files and input file format specification.
     * It is a drop-in replacement for {@link java.util.Scanner} that could be added to a solution source
     * (cut-and-paste) without breaking its ability to work with {@link java.util.Scanner}.
     */
    public class Scanner {
        private final BufferedReader in;
        private String line = "";
        private int pos;
        private int lineNo;

        public Scanner(File source) throws FileNotFoundException {
            in = new BufferedReader(new FileReader(source));
            nextLine();
        }

        public void close() {
            assert line == null : "Extra data at the end of file";
            try {
                in.close();
            } catch (IOException e) {
                throw new AssertionError("Failed to close with " + e);
            }
        }

        public void nextLine() {
            assert line != null : "EOF";
            assert pos == line.length() : "Extra characters on line " + lineNo;
            try {
                line = in.readLine();
            } catch (IOException e) {
                throw new AssertionError("Failed to read line with " + e);
            }
            pos = 0;
            lineNo++;
        }

        public String next() {
            assert line != null : "EOF";
            assert line.length() > 0 : "Empty line " + lineNo;
            if (pos == 0)
                assert line.charAt(0) > ' ' : "Line " + lineNo + " starts with whitespace";
            else {
                assert pos < line.length() : "Line " + lineNo + " is over";
                assert line.charAt(pos) == ' ' : "Wrong whitespace on line " + lineNo;
                pos++;
                assert pos < line.length() : "Line " + lineNo + " is over";
                assert line.charAt(pos) > ' ' : "Line " + lineNo + " has double whitespace";
            }
            StringBuilder sb = new StringBuilder();
            while (pos < line.length() && line.charAt(pos) > ' ')
                sb.append(line.charAt(pos++));
            return sb.toString();
        }

        public int nextInt() {
            String s = next();
            assert s.length() == 1 || s.charAt(0) != '0' : "Extra leading zero in number " + s + " on line " + lineNo;
            assert s.charAt(0) != '+' : "Extra leading '+' in number " + s + " on line " + lineNo;
            try {
                return Integer.parseInt(s);
            } catch (NumberFormatException e) {
                throw new AssertionError("Malformed number " + s + " on line " + lineNo);
            }
        }

        public long nextLong() {
            String s = next();
            assert s.length() == 1 || s.charAt(0) != '0' : "Extra leading zero in number " + s + " on line " + lineNo;
            assert s.charAt(0) != '+' : "Extra leading '+' in number " + s + " on line " + lineNo;
            try {
                return Long.parseLong(s);
            } catch (NumberFormatException e) {
                throw new AssertionError("Malformed number " + s + " on line " + lineNo);
            }
        }
    }
}

