/******************************************************************/
/* ACM ICPC 2012-2013                                             */
/* Northeastern European Regional Contest                         */
/* St Petersburg - Barnaul - Tashkent - Tbilisi, December 2, 2012 */
/******************************************************************/
/* Problem I. Identification of Protein                           */
/*                                                                */
/* Original idea         Mikhail Dvorkin                          */
/* Problem statement     Mikhail Dvorkin                          */
/* Test set              Mikhail Dvorkin                          */
/******************************************************************/
/* Tests                                                          */
/*                                                                */
/* Author                Mikhail Dvorkin                          */
/******************************************************************/

import java.io.*;
import java.util.*;

public class Tests {
	private static File dir = new File("tests");
	private int curTest = 1;
	private static boolean verbose = true;

	private String testName;
	private PrintWriter out;
	Random r = new Random(36716685609813687L);

	static final long T = 100000;
	static final long P = 9705276;
	static final long Q = 12805858;
	static final long[] M = new long[]{P, Q};

	static final int MAX_PEAKS = 100000;
	static final int MAX_LEN = 400;
	
	public void run() {
        t(p(1, 1), p(1, 2), p(1, 2) - 1, p(3, 0), p(1, 0), p(0, 1));
		t(p(1, 0), p(3, 0), p(2, 0));
		t(p(1, 0), p(0, 1), p(1, 1));
		t(p(1, 0));
		t(p(0, 4));
		t(p(1, 3), p(0, 4), p(2, 2), p(4, 0), p(3, 1));
		t(new Spectrum(50, 5, true).addCorrectPeaks(1));
		ts(new Spectrum(101, 100, true, 1000).addCorrectPeaks(0.9));
		ts(new Spectrum(MAX_LEN, 0, true).addCorrectPeaks(0.1).addRandomNoiseUpTo(MAX_PEAKS));
		ts(new Spectrum(MAX_LEN, MAX_LEN / 2, true).addCorrectPeaks(0.5).addRandomNoiseUpTo(MAX_PEAKS));
		ts(new Spectrum(MAX_LEN, 7, true).addCorrectPeaks(0.5).addRandomNoiseUpTo(MAX_PEAKS));
		t(new Spectrum(1, 0).addRandomNoiseUpTo(MAX_PEAKS));
		
		for (int p : new int[]{0, 1, 13, MAX_LEN - 1, MAX_LEN}) {
			int q = MAX_LEN - p;
			long mass = p(p, q);
			Set<Long> peaks = new TreeSet<Long>();
			peaks.add(mass);
			for (int i = 0; p(i, 0) < mass; i++) {
				for (int j = 0; p(i, j) < mass; j++) {
					peaks.add(p(i, j));
				}
			}
			peaks.remove(0L);
			List<Long> list = new ArrayList<>(peaks);
			ts(list.subList(Math.max(0, list.size() - MAX_PEAKS), list.size()));
		}
		
		for (int m : new int[]{30, MAX_LEN / 2}) {
			int[][] matrix = new int[m + 1][m + 1];
			fillRect(matrix, 0, 0, m, m, 1);
			ts(new Spectrum(2 * m, m, matrix));
			
			fillRect(matrix, 0, 0, m + 1, m + 1, 1);
			ts(new Spectrum(2 * m, m, matrix));
			
			ts(new Spectrum(2 * m, 1, matrix));
			
			fillCircle(matrix,  m / 4, m / 4, m / 4, 2);
			ts(new Spectrum(2 * m, m, matrix));
			
			fillRect(matrix, 0, 0, m + 1, m + 1, 1);
			for (int i = 0; i <= m; i++) {
				int j = i / 2 + (int) (5 * Math.sin(i * 0.5));
				if (i + j >= m) {
					break;
				}
				matrix[i][j] = 2;
			}
			ts(new Spectrum(2 * m - 1, m, matrix));
			
			fillRect(matrix, 0, 0, m + 1, m + 1, 1);
			for (int i = 0; i <= m; i++) {
				int j = i + (int) (4 * Math.sin(i * 0.2));
				if (i + j >= m) {
					break;
				}
				matrix[i][j] = 2;
			}
			ts(new Spectrum(2 * m - 1, 4 * m / 3, matrix).addRandomNoiseUpTo(MAX_PEAKS));
			for (int i = 0; i <= m; i++) {
				int j = i + (int) (4 * Math.cos(i * 0.2));
				if (i + j >= m) {
					break;
				}
				matrix[i][j] = 2;
			}
			ts(new Spectrum(2 * m - 1, m, matrix).addRandomNoiseUpTo(MAX_PEAKS));
			
			fillRect(matrix, 0, 0, m + 1, m + 1, 0);
			for (int i = 0; i <= m; i++) {
				for (int j = 0; j <= m; j++) {
					if (i % 7 == 3 || j % 7 == 3) {
						matrix[i][j] = 1;
					}
				}
			}
			ts(new Spectrum(2 * m - 1, 3 * m / 4, matrix).addRandomNoiseUpTo(MAX_PEAKS));
			
			for (int i = 0; i <= m; i++) {
				for (int j = 0; j <= m; j++) {
					if (Math.round(Math.sqrt(i)) == j || Math.round(Math.sqrt(j)) == i) {
						matrix[i][j] = 2;
					}
				}
			}
			ts(new Spectrum(2 * m - 1, 3 * m / 4, matrix).addRandomNoiseUpTo(MAX_PEAKS));

			fillRect(matrix, 0, 0, m + 1, m + 1, 1);
			fillCircle(matrix, m, 0, m, 2);
			fillCircle(matrix, 0, m, m, 2);
			ts(new Spectrum(2 * m - 1, m / 2 - 3, matrix).addRandomNoiseUpTo(MAX_PEAKS));
			ts(new Spectrum(2 * m, 2 * m, matrix).addRandomNoiseUpTo(MAX_PEAKS));
			ts(new Spectrum(3 * m / 2 + 1, m / 2, matrix).addRandomNoiseUpTo(MAX_PEAKS));
			ts(new Spectrum(2 * m - 1, 2 * m - 1, true).addCorrectPeaks(1).addRandomNoiseUpTo(MAX_PEAKS));
			ts(new Spectrum(2 * m, m, true).addCorrectPeaks(1).addRandomNoiseUpTo(MAX_PEAKS));
			ts(new Spectrum(2 * m, 0, true).addCorrectPeaks(1).addRandomNoiseUpTo(MAX_PEAKS));
        }
        for (int size : new int[]{6, 10}) {
            ts(buildTrickyCase(size, size, 1.0, 1.0));
            ts(buildTrickyCase(size, size, 1.0, 0.5));
            ts(buildTrickyCase(size, size, 0.5, 1.0));
            ts(buildTrickyCase(size, size, 0.9, 0.9));
        }
        ts(buildTrickyCase(100, 100, 1.0, 1.0));
        ts(buildTrickyCase(200, 200, 1.0, 1.0));
        for (int i = 0; i < 10; ++i) {
            ts(buildTrickyCase2(10, 10, 1.0, 1.0));
        }

        System.out.println("Generating big tests against petr_wa_bug2 (close tests - 1 off)");
        new StressTestPetrBug2().generate();
    }

    private Spectrum buildTrickyCase(int maxp, int maxq, double prBoth, double prOnly) {
        while (true) {
            int p = r.nextInt(maxp) + 1;
            int q = r.nextInt(maxq) + 1;
            if (p == q || ((p + q) % 2 != 0))
                continue;
            int n = p + q;
            Spectrum s1 = new Spectrum(n, p);
            s1.setRandomProtein();
            s1.addCorrectPeaks(1.0);
            Spectrum s2 = new Spectrum(n, p);
            s2.setRandomProtein();
            s2.addCorrectPeaks(1.0);
            Set<Long> both = new TreeSet<>(s1.peaks);
            both.retainAll(s2.peaks);
            Set<Long> s1Only = new TreeSet<>(s1.peaks);
            s1Only.removeAll(s2.peaks);
            Set<Long> s2Only = new TreeSet<>(s2.peaks);
            s2Only.removeAll(s1.peaks);
            long middlePeak1 = s1.cumulative[n / 2];
            long revMiddlePeak1 = s1.mass - middlePeak1;
            long middlePeak2 = s2.cumulative[n / 2];
            long revMiddlePeak2 = s2.mass - middlePeak2;
            if (!s1Only.contains(middlePeak1) || !s1Only.contains(revMiddlePeak1) || middlePeak1 == revMiddlePeak1) continue;
            if (!s2Only.contains(middlePeak2) || !s2Only.contains(revMiddlePeak2) || middlePeak2 == revMiddlePeak2) continue;
            Spectrum res = new Spectrum(n, p);
            int numBoth = (int) Math.round(both.size() * prBoth);
            res.peaks.addAll(getRandomSubset(both, numBoth));
            s1Only.remove(middlePeak1);
            s1Only.remove(revMiddlePeak1);
            s2Only.remove(middlePeak2);
            s2Only.remove(revMiddlePeak2);
            int extra = Math.min(s1Only.size(), s2Only.size() - 3);
            if (extra <= 0) continue;
            int numOnly1 = (int) Math.round(extra * prOnly);
            int numOnly2 = numOnly1 + 3;
            res.peaks.addAll(getRandomSubset(s1Only, numOnly1));
            res.peaks.addAll(getRandomSubset(s2Only, numOnly2));
            res.peaks.add(middlePeak1);
            res.peaks.add(revMiddlePeak1);
            Set<Long> expected = new TreeSet<>(res.peaks);
            expected.retainAll(s2.peaks);
            Set<Long> expected2 = new TreeSet<>(res.peaks);
            expected2.retainAll(s1.peaks);
            if (!Solution.checkSolutionCatchesBad(res.peaks, n, expected.size())) continue;
            return res;
        }
    }

    private Spectrum buildTrickyCase2(int maxp, int maxq, double prBoth, double prOnly) {
        while (true) {
            int p = r.nextInt(maxp - 1) + 2;
            int q = r.nextInt(maxq - 1) + 2;
            if (p % 2 != 0) {
                --p;
            }
            if (q % 2 != 0) {
                --q;
            }
            if (p == q || ((p + q) % 2 != 0))
                continue;
            int n = p + q;
            Spectrum s1 = new Spectrum(n, p);
            s1.setRandomProteinWithExactHalf();
            s1.addCorrectPeaks(1.0);
            Spectrum s2 = new Spectrum(n, p);
            s2.setRandomProtein();
            s2.addCorrectPeaks(1.0);
            Set<Long> both = new TreeSet<>(s1.peaks);
            both.retainAll(s2.peaks);
            Set<Long> s1Only = new TreeSet<>(s1.peaks);
            s1Only.removeAll(s2.peaks);
            Set<Long> s2Only = new TreeSet<>(s2.peaks);
            s2Only.removeAll(s1.peaks);
            long middlePeak1 = s1.cumulative[n / 2];
            long revMiddlePeak1 = s1.mass - middlePeak1;
            if (middlePeak1 != revMiddlePeak1) throw new RuntimeException();
            long middlePeak2 = s2.cumulative[n / 2];
            long revMiddlePeak2 = s2.mass - middlePeak2;
            if (middlePeak2 == revMiddlePeak2) continue;
            Spectrum res = new Spectrum(n, p);
            int numBoth = (int) Math.round(both.size() * prBoth);
            res.peaks.addAll(getRandomSubset(both, numBoth));
            int extra = Math.min(s1Only.size() - 1, s2Only.size());
            if (extra <= 0) continue;
            int numOnly2 = (int) Math.round(extra * prOnly);
            int numOnly1 = numOnly2 - 1;
            res.peaks.addAll(getRandomSubset(s1Only, numOnly1));
            res.peaks.addAll(getRandomSubset(s2Only, numOnly2));
            Set<Long> expected = new TreeSet<>(res.peaks);
            expected.retainAll(s2.peaks);
            Set<Long> expected2 = new TreeSet<>(res.peaks);
            expected2.retainAll(s1.peaks);
            if (!Solution.checkSolutionCatchesBad2(res.peaks, n, expected.size())) continue;
            return res;
        }
    }


    private Set<Long> getRandomSubset(Collection<Long> values, int numTake) {
        List<Long> l = new ArrayList<>(values);
        Collections.shuffle(l, r);
        return new TreeSet<>(l.subList(0, numTake));
    }

    class Spectrum {
		final int n, p, q;
		final long mass;
		TreeSet<Long> peaks = new TreeSet<>();
		
		int[] protein;
		long[] cumulative;

        @Override public String toString() {
            String res = "";
            for (int x : protein) if (x == 0) res += "P"; else res += "Q";
            return res;
        }
		
		public Spectrum(int n, int p) {
			this.n = n;
			this.p = p;
			q = n - p;
			mass = p(p, q);
			peaks.add(mass);
		}

		public Spectrum(int n, int p, int[][] matrix) {
			this(n, p);
			for (int i = 0; i < matrix.length && i <= n / 2; i++) {
				for (int j = 0; j < matrix[i].length && i + j <= n / 2; j++) {
					if (matrix[i][j] == 0) {
						continue;
					}
					long peak = p(i, j);
					if (peak == 0 || peak >= mass) {
						continue;
					}
					if (matrix[i][j] == 1) {
						peaks.add(r.nextBoolean() ? peak : (mass - peak));
					} else {
						peaks.add(peak);
						peaks.add(mass - peak);
					}
				}
			}
		}

		public Spectrum(int n, int p, boolean randomProtein) {
			this(n, p);
			if (randomProtein) {
				setRandomProtein();
			}
		}

		public Spectrum(int n, int p, boolean randomProtein, int noiseAmount) {
			this(n, p, randomProtein);
			addRandomNoise(noiseAmount);
		}

		public Spectrum calcCumulative() {
			if (protein == null) throw new IllegalStateException();
			cumulative = new long[n + 1];
			for (int i = 0; i < n; i++) {
				cumulative[i + 1] = cumulative[i] + M[protein[i]];
			}
			return this;
		}
		
		public Spectrum addCorrectPeaks(double prob) {
			if (protein == null) throw new IllegalStateException();
			calcCumulative();
			for (int i = 1; i < n; i++) {
				if (r.nextDouble() < prob) {
					peaks.add(cumulative[i]);
				}
				if (r.nextDouble() < prob) {
					peaks.add(cumulative[n] - cumulative[n - i]);
				}
			}
			return this;
		}
		
		public Spectrum addRandomNoiseUpTo(int amount) {
			return addRandomNoise(amount - peaks.size());
		}
		
		public Spectrum addRandomNoise(int amount) {
			amount += peaks.size();
			while (peaks.size() < amount) {
				long peak = Math.round(r.nextDouble() * mass);
				if (peak == 0) {
					continue;
				}
				peaks.add(peak);
			}
			return this;
		}
		
		public Spectrum setRandomProtein() {
			List<Integer> list = new ArrayList<Integer>();
			for (int i = 0; i < n; i++) {
				list.add((i < p) ? 0 : 1);
			}
			Collections.shuffle(list, r);
			protein = new int[n];
			for (int i = 0; i < n; i++) {
				protein[i] = list.get(i);
			}
			return this;
		}

        public void setRandomProteinWithExactHalf() {
            List<Integer> list = new ArrayList<Integer>();
            for (int i = 0; i < n; i++) {
                list.add((i < p / 2 || (i >= p / 2 + q / 2 && i < p + q / 2)) ? 0 : 1);
            }
            Collections.shuffle(list.subList(0, p / 2 + q / 2), r);
            Collections.shuffle(list.subList(p / 2 + q / 2, list.size()), r);
            protein = new int[n];
            for (int i = 0; i < n; i++) {
                protein[i] = list.get(i);
            }
        }
    }
	
	private static void fillRect(int[][] matrix, int iFrom, int jFrom, int iTo, int jTo, int value) {
		for (int i = iFrom; i < iTo; i++) {
			for (int j = jFrom; j < jTo; j++) {
				matrix[i][j] = value;
			}
		}
	}

	private static void fillCircle(int[][] matrix, int iCenter, int jCenter, int r, int value) {
		int c = 8 * r;
		for (int k = 0; k < c; k++) {
			double angle = 2 * Math.PI * k / c;
			int i = iCenter + (int) Math.round(r * Math.cos(angle));
			int j = jCenter + (int) Math.round(r * Math.sin(angle));
			if (i >= 0 && j >= 0 && i < matrix.length && j < matrix[i].length) {
				matrix[i][j] = value;
			}
		}
	}

	public static long p(int p, int q) {
		return p * P + q * Q;
	}

	private static long[] toLongArray(Collection<Long> peaks) {
		long[] result = new long[peaks.size()];
		int i = 0;
		for (Long peak : peaks) {
			result[i] = peak;
			i++;
		}
		return result;
	}
	
	private void t(Spectrum spectrum) {
		t(spectrum.peaks);
	}
	
	private void ts(Spectrum spectrum) {
		ts(spectrum.peaks);
	}
	
	private void ts(Collection<Long> peaks) {
		List<Long> list = new ArrayList<>(peaks);
		Collections.shuffle(list, r);
		t(list);
	}
	
	private void t(Collection<Long> peaks) {
		t(toLongArray(peaks));
	}
	
	private void t(long... peaks) {
		open();
		out.println(peaks.length);
		for (long peak : peaks) {
			out.printf(Locale.US, "%.5f%n", peak * 1.0 / T);
		}
		close();
	}

	private void open() {
		try {
			testName = "" + curTest / 10 + curTest % 10;
			out = new PrintWriter(new File(dir, testName));
			if (verbose) { 
				System.out.print("Generating " + testName + "...");
			}
		} catch (IOException e) {
			throw new RuntimeException("Unable to open " + testName + " for writing", e);
		}
	}

	private void close() {
		out.close();
		if (verbose) {
			System.out.println(" done.");
		}
		curTest++;
	}

	public static void main(String[] args) {
		Locale.setDefault(Locale.US);
		dir.mkdirs();
		new Tests().run();
	}

    private static class Solution {
        public static boolean checkSolutionCatchesBad(TreeSet<Long> peakSet, int MAX_SIZE, int expectedAnswer) {
            return solveCorrectly(peakSet, MAX_SIZE) == expectedAnswer && solveIncorrectly(peakSet, MAX_SIZE) == expectedAnswer + 1;
        }

        private static int solveCorrectly(TreeSet<Long> peakSet, int MAX_SIZE) {
            long[] peaks = new long[peakSet.size()];
            int ptr = 0;
            for (long x : peakSet) peaks[ptr++] = x;
            Arrays.sort(peaks);
            int[][] have = new int[MAX_SIZE + 1][MAX_SIZE + 1];
            int mp = 0;
            int mq = 0;
            for (int p = 0; p <= MAX_SIZE; ++p)
                for (int q = 0; p + q <= MAX_SIZE; ++q)
                    if (Arrays.binarySearch(peaks, p * P + q * Q) >= 0) {
                        have[p][q] = 1;
                        if (p * P + q * Q > mp * P + mq * Q) {
                            mp = p;
                            mq = q;
                        }
                    }
            int[][] nhave = new int[mp + 1][mq + 1];
            for (int p = 0; p <= mp; ++p)
                for (int q = 0; q <= mq; ++q) {
                    nhave[p][q] = have[p][q];
                    if (p != mp - p || q != mq - q)
                        nhave[p][q] += have[mp - p][mq - q];
                }
            int steps = (mp + mq) / 2;
            int[][] best = new int[1][1];
            for (int step = 0; step <= steps; ++step) {
                int[][] nbest = new int[step + 2][step + 2];
                for (int[] x : nbest) Arrays.fill(x, -1);
                for (int cl = 0; cl <= step; ++cl)
                    for (int cr = 0; cr <= step; ++cr) {
                        int nl = cl;
                        int nr = cr;
                        int bonus = 0;
                        if (nl <= mp && step - nl <= mq) bonus += nhave[nl][step - nl];
                        if (nl != nr && step * 2 != mp + mq) {
                            if (nr <= mp && step - nr <= mq)
                                bonus += nhave[nr][step - nr];
                        }
                        best[cl][cr] += bonus;
                        if (step < steps)
                            for (int sl = 0; sl < 2; ++sl)
                                for (int sr = 0; sr < 2; ++sr) {
                                    nl = cl + sl;
                                    nr = cr + sr;
                                    if (nbest[nl][nr] < best[cl][cr]) {
                                        nbest[nl][nr] = best[cl][cr];
                                    }
                                }
                    }
                if (step < steps) {
                    best = nbest;
                }
            }
            int resAns = -1;
            for (int cl = 0; cl <= steps; ++cl)
                for (int cr = 0; cr <= steps; ++cr) {
                    int dl = steps - cl;
                    int dr = steps - cr;
                    if (cl + cr <= mp && dl + dr <= mq) {
                        if (best[cl][cr] > resAns) {
                            resAns = best[cl][cr];
                        }
                    }
                }
            return resAns;
        }

        private static int solveIncorrectly(TreeSet<Long> peakSet, int MAX_SIZE) {
            long[] peaks = new long[peakSet.size()];
            int ptr = 0;
            for (long x : peakSet) peaks[ptr++] = x;
            Arrays.sort(peaks);
            int[][] have = new int[MAX_SIZE + 1][MAX_SIZE + 1];
            int mp = 0;
            int mq = 0;
            for (int p = 0; p <= MAX_SIZE; ++p)
                for (int q = 0; p + q <= MAX_SIZE; ++q)
                    if (Arrays.binarySearch(peaks, p * P + q * Q) >= 0) {
                        have[p][q] = 1;
                        if (p * P + q * Q > mp * P + mq * Q) {
                            mp = p;
                            mq = q;
                        }
                    }
            int[][] nhave = new int[mp + 1][mq + 1];
            for (int p = 0; p <= mp; ++p)
                for (int q = 0; q <= mq; ++q) {
                    nhave[p][q] = have[p][q];
                    if (p != mp - p || q != mq - q)
                        nhave[p][q] += have[mp - p][mq - q];
                }
            int steps = (mp + mq) / 2;
            int[][] best = new int[1][1];
            for (int step = 0; step <= steps; ++step) {
                int[][] nbest = new int[step + 2][step + 2];
                for (int[] x : nbest) Arrays.fill(x, -1);
                for (int cl = 0; cl <= step; ++cl)
                    for (int cr = 0; cr <= step; ++cr) {
                        int nl = cl;
                        int nr = cr;
                        int bonus = 0;
                        if (nl <= mp && step - nl <= mq) bonus += nhave[nl][step - nl];
                        if (nl != nr) {
                            if (nr <= mp && step - nr <= mq)
                                bonus += nhave[nr][step - nr];
                        }
                        best[cl][cr] += bonus;
                        if (step < steps)
                            for (int sl = 0; sl < 2; ++sl)
                                for (int sr = 0; sr < 2; ++sr) {
                                    nl = cl + sl;
                                    nr = cr + sr;
                                    if (nbest[nl][nr] < best[cl][cr]) {
                                        nbest[nl][nr] = best[cl][cr];
                                    }
                                }
                    }
                if (step < steps) {
                    best = nbest;
                }
            }
            int resAns = -1;
            for (int cl = 0; cl <= steps; ++cl)
                for (int cr = 0; cr <= steps; ++cr) {
                    int dl = steps - cl;
                    int dr = steps - cr;
                    if (cl + cr <= mp && dl + dr <= mq) {
                        if (best[cl][cr] > resAns) {
                            resAns = best[cl][cr];
                        }
                    }
                }
            return resAns;
        }

        private static int solveIncorrectly2(TreeSet<Long> peakSet, int MAX_SIZE) {
            long[] peaks = new long[peakSet.size()];
            int ptr = 0;
            for (long x : peakSet) peaks[ptr++] = x;
            Arrays.sort(peaks);
            int[][] have = new int[MAX_SIZE + 1][MAX_SIZE + 1];
            int mp = 0;
            int mq = 0;
            for (int p = 0; p <= MAX_SIZE; ++p)
                for (int q = 0; p + q <= MAX_SIZE; ++q)
                    if (Arrays.binarySearch(peaks, p * P + q * Q) >= 0) {
                        have[p][q] = 1;
                        if (p * P + q * Q > mp * P + mq * Q) {
                            mp = p;
                            mq = q;
                        }
                    }
            int[][] nhave = new int[mp + 1][mq + 1];
            boolean[][] ntained = new boolean[mp + 1][mq + 1];
            for (int p = 0; p <= mp; ++p)
                for (int q = 0; q <= mq; ++q) {
                    nhave[p][q] = have[p][q];
                    nhave[p][q] += have[mp - p][mq - q];
                    if (!(p != mp - p || q != mq - q)) {
                        ntained[p][q] = true;
                    }
                }
            int steps = (mp + mq) / 2;
            int[][] best = new int[1][1];
            boolean[][] btained = new boolean[1][1];
            for (int step = 0; step <= steps; ++step) {
                int[][] nbest = new int[step + 2][step + 2];
                boolean[][] nbtainted = new boolean[step + 2][step + 2];
                for (int[] x : nbest) Arrays.fill(x, -1);
                for (int cl = 0; cl <= step; ++cl)
                    for (int cr = 0; cr <= step; ++cr) {
                        int nl = cl;
                        int nr = cr;
                        int bonus = 0;
                        if (nl <= mp && step - nl <= mq) {
                            bonus += nhave[nl][step - nl];
                            if (ntained[nl][step - nl]) btained[cl][cr] = true;
                        }
                        if (nl != nr) {
                            if (nr <= mp && step - nr <= mq)
                                bonus += nhave[nr][step - nr];
                        }
                        best[cl][cr] += bonus;
                        if (step < steps)
                            for (int sl = 0; sl < 2; ++sl)
                                for (int sr = 0; sr < 2; ++sr) {
                                    nl = cl + sl;
                                    nr = cr + sr;
                                    if (nbest[nl][nr] < best[cl][cr]) {
                                        nbest[nl][nr] = best[cl][cr];
                                    }
                                }
                    }
                if (step < steps) {
                    best = nbest;
                    btained = nbtainted;
                }
            }
            int resAns = -1;
            boolean resTainted = false;
            for (int cl = 0; cl <= steps; ++cl)
                for (int cr = 0; cr <= steps; ++cr) {
                    int dl = steps - cl;
                    int dr = steps - cr;
                    if (cl + cr <= mp && dl + dr <= mq) {
                        if (best[cl][cr] > resAns) {
                            resAns = best[cl][cr];
                            resTainted = false;
                        }
                        if (best[cl][cr] == resAns && btained[cl][cr])
                            resTainted = true;
                    }
                }
            return resTainted ? (-resAns) : resAns;
        }


        public static boolean checkSolutionCatchesBad2(TreeSet<Long> peakSet, int MAX_SIZE, int expectedAnswer) {
            return solveCorrectly(peakSet, MAX_SIZE) == expectedAnswer && solveIncorrectly2(peakSet, MAX_SIZE) == -expectedAnswer;
        }
    }

    public class StressTestPetrBug2 {
        static final long peakP = 9705276;
        static final long peakQ = 12805858;

        static final int MAX_SIZE = 200;

        static final int N_STOP = 10;

        public void generate() {
            Random r = new Random(20121201);
            int nFound = 0;
            for (int num = 1; ; num++) {
                PetrBug2Test t;
                int cnt;
                do {
                    t = new PetrBug2Test();
                    do {
                        double prob = r.nextDouble();
                        cnt = 0;
                        for (int p = 0; p <= MAX_SIZE; p++)
                            for (int q = 0; q <= MAX_SIZE; q++)
                                if ((p > 0 || q > 0) && r.nextDouble() > prob) {
                                    t.have[p][q] = 1;
                                    cnt++;
                                }
                    } while (cnt == 0);
                    t.initMax();
                } while (t.mp % 2 != 0 || t.mq %2 != 0);
                String s1 = t.solveBug2();
                String s2 = t.solveCorrect();
                int e1 = t.explains(s1);
                int e2 = t.explains(s2);
                if (e1 != e2) {
                    // FOUND
                    List<Long> peaks = new ArrayList<>();
                    for (int p = 0; p <= MAX_SIZE; p++)
                        for (int q = 0; q <= MAX_SIZE; q++)
                            if (t.have[p][q] > 0) {
                                peaks.add(p * peakP + q * peakQ);
                            }
                    ts(peaks);
                    nFound++;
                    if (nFound >= N_STOP)
                        return;
                }
            }
        }

        class PetrBug2Test {
            int[][] have = new int[MAX_SIZE + 1][MAX_SIZE + 1];

            public int explains(String s) {
                boolean[][] v = new boolean[MAX_SIZE + 1][MAX_SIZE + 1];
                int cnt = 0;
                int p = 0;
                int q = 0;
                for (int i = 0; i < s.length() - 1; i++) {
                    switch (s.charAt(i)) {
                        case 'P':
                            p++;
                            break;
                        case 'Q':
                            q++;
                            break;
                        default:
                            throw new AssertionError();
                    }
                    v[p][q] = true;
                    cnt += have[p][q];
                }
                p = 0;
                q = 0;
                for (int i = s.length(); --i > 0;) {
                    switch (s.charAt(i)) {
                        case 'P':
                            p++;
                            break;
                        case 'Q':
                            q++;
                            break;
                        default:
                            throw new AssertionError();
                    }
                    if (!v[p][q])
                        cnt += have[p][q];
                }
                return cnt;
            }

            private int countPrefix(String s) {
                int cnt = 0;
                int p = 0;
                int q = 0;
                for (int i = 0; i < s.length(); i++) {
                    switch (s.charAt(i)) {
                        case 'P':
                            p++;
                            break;
                        case 'Q':
                            q++;
                            break;
                        default:
                            throw new AssertionError();
                    }
                    cnt += have[p][q];
                }
                return cnt;
            }

            int mp;
            int mq;

            public void initMax() {
                mp = 0;
                mq = 0;
                for (int p = 0; p <= MAX_SIZE; ++p)
                    for (int q = 0; q <= MAX_SIZE; ++q)
                        if (have[p][q] > 0) {
                            if (p * peakP + q * peakQ > mp * peakP + mq * peakQ) {
                                mp = p;
                                mq = q;
                            }
                        }
            }

            public String solveBug2() {
                int[][] nhave = new int[mp + 1][mq + 1];
                for (int p = 0; p <= mp; ++p)
                    for (int q = 0; q <= mq; ++q) {
                        nhave[p][q] = have[p][q];
                        nhave[p][q] += have[mp - p][mq - q];
                    }
                int steps = (mp + mq) / 2;
                byte[][][] prev = new byte[steps + 1][][];
                int[][] best = new int[1][1];
                for (int step = 0; step <= steps; ++step) {
                    prev[step] = new byte[step + 2][step + 2];
                    int[][] nbest = new int[step + 2][step + 2];
                    for (int[] x : nbest) Arrays.fill(x, -1);
                    for (int cl = 0; cl <= step; ++cl)
                        for (int cr = 0; cr <= step; ++cr) {
                            int nl = cl;
                            int nr = cr;
                            int bonus = 0;
                            if (nl <= mp && step - nl <= mq) bonus += nhave[nl][step - nl];
                            if (nl != nr && step * 2 != mp + mq) {
                                if (nr <= mp && step - nr <= mq)
                                    bonus += nhave[nr][step - nr];
                            }
                            best[cl][cr] += bonus;
                            if (step < steps)
                                for (int sl = 0; sl < 2; ++sl)
                                    for (int sr = 0; sr < 2; ++sr) {
                                        nl = cl + sl;
                                        nr = cr + sr;
                                        if (nbest[nl][nr] < best[cl][cr]) {
                                            nbest[nl][nr] = best[cl][cr];
                                            prev[step][nl][nr] = (byte) (sl * 2 + sr);
                                        }
                                    }
                        }
                    if (step < steps) best = nbest;
                }
                int resAns = -1;
                int rcl = -1;
                int rcr = -1;
                String extra = "";
                for (int cl = 0; cl <= steps && 2 * cl <= mp; ++cl)
                    for (int cr = 0; cr <= steps; ++cr) {
                        int dl = steps - cl;
                        int dr = steps - cr;
                        if (cl + cr <= mp && dl + dr <= mq) {
                            if (best[cl][cr] >= resAns) {
                                resAns = best[cl][cr];
                                rcl = cl;
                                rcr = cr;
                                if (cl + cr < mp) extra = "P"; else if (dl + dr < mq) extra = "Q"; else extra = "";
                            }
                        }
                    }
                String prefix = "";
                String suffix = "";
                for (int step = steps - 1; step >= 0; --step) {
                    int by = prev[step][rcl][rcr];
                    if (by / 2 > 0) {
                        prefix = "P" + prefix;
                        --rcl;
                    } else {
                        prefix = "Q" + prefix;
                    }
                    if (by % 2 > 0) {
                        suffix += "P";
                        --rcr;
                    } else {
                        suffix += "Q";
                    }
                }
                return prefix + extra + suffix;
            }

            public String solveCorrect() {
                int[][] nhave = new int[mp + 1][mq + 1];
                for (int p = 0; p <= mp; ++p)
                    for (int q = 0; q <= mq; ++q) {
                        nhave[p][q] = have[p][q];
                        if (p != mp - p || q != mq - q)
                            nhave[p][q] += have[mp - p][mq - q];
                    }
                int steps = (mp + mq) / 2;
                byte[][][] prev = new byte[steps + 1][][];
                int[][] best = new int[1][1];
                for (int step = 0; step <= steps; ++step) {
                    prev[step] = new byte[step + 2][step + 2];
                    int[][] nbest = new int[step + 2][step + 2];
                    for (int[] x : nbest) Arrays.fill(x, -1);
                    for (int cl = 0; cl <= step; ++cl)
                        for (int cr = 0; cr <= step; ++cr) {
                            int nl = cl;
                            int nr = cr;
                            int bonus = 0;
                            if (nl <= mp && step - nl <= mq) bonus += nhave[nl][step - nl];
                            if (nl != nr && step * 2 != mp + mq) {
                                if (nr <= mp && step - nr <= mq)
                                    bonus += nhave[nr][step - nr];
                            }
                            best[cl][cr] += bonus;
                            if (step < steps)
                                for (int sl = 0; sl < 2; ++sl)
                                    for (int sr = 0; sr < 2; ++sr) {
                                        nl = cl + sl;
                                        nr = cr + sr;
                                        if (nbest[nl][nr] < best[cl][cr]) {
                                            nbest[nl][nr] = best[cl][cr];
                                            prev[step][nl][nr] = (byte) (sl * 2 + sr);
                                        }
                                    }
                        }
                    if (step < steps) best = nbest;
                }
                int resAns = -1;
                int rcl = -1;
                int rcr = -1;
                String extra = "";
                for (int cl = 0; cl <= steps; ++cl)
                    for (int cr = 0; cr <= steps; ++cr) {
                        int dl = steps - cl;
                        int dr = steps - cr;
                        if (cl + cr <= mp && dl + dr <= mq) {
                            if (best[cl][cr] > resAns) {
                                resAns = best[cl][cr];
                                rcl = cl;
                                rcr = cr;
                                if (cl + cr < mp) extra = "P"; else if (dl + dr < mq) extra = "Q"; else extra = "";
                            }
                        }
                    }
                String prefix = "";
                String suffix = "";
                for (int step = steps - 1; step >= 0; --step) {
                    int by = prev[step][rcl][rcr];
                    if (by / 2 > 0) {
                        prefix = "P" + prefix;
                        --rcl;
                    } else {
                        prefix = "Q" + prefix;
                    }
                    if (by % 2 > 0) {
                        suffix += "P";
                        --rcr;
                    } else {
                        suffix += "Q";
                    }
                }
                return prefix + extra + suffix;
            }
        }
    }
}

