/******************************************************************/
/* ACM ICPC 2012-2013                                             */
/* Northeastern European Regional Contest                         */
/* St Petersburg - Barnaul - Tashkent - Tbilisi, December 2, 2012 */
/******************************************************************/
/* Problem I. Identification of Protein                           */
/*                                                                */
/* Original idea         Mikhail Dvorkin                          */
/* Problem statement     Mikhail Dvorkin                          */
/* Test set              Mikhail Dvorkin                          */
/******************************************************************/
/* Solution                                                       */
/*                                                                */
/* Author                Mikhail Dvorkin                          */
/******************************************************************/

import java.awt.Point;
import java.io.*;
import java.util.*;

public class identification_md {
	private static String fileName = identification_md.class.getSimpleName().replaceFirst("_.*", "").toLowerCase();
	private static String inputFileName = fileName + ".in";
	private static String outputFileName = fileName + ".out";
	private static BufferedReader in;
	private static PrintWriter out;

	static final long T = 100000;
	static final long P = 9705276;
	static final long Q = 12805858;
	
	static final int MAX_LEN = 500;
	static final Map<Long, Point> peakMap = new HashMap<Long, Point>();
	static {
		for (int p = 0; p <= MAX_LEN; p++) {
			for (int q = 0; p + q <= MAX_LEN; q++) {
				peakMap.put(p * P + q * Q, new Point(p, q));
			}
		}
	}
	
	public void run() throws IOException {
		List<Point> peaks = new ArrayList<Point>();
		Point total = new Point();
		long totalMass = 0;
		for (int i = Integer.parseInt(in.readLine()); i > 0; i--) {
			long mass = Math.round(Double.parseDouble(in.readLine()) * T);
			Point peak = peakMap.get(mass);
			if (peak == null) {
				continue;
			}
			peaks.add(peak);
			if (mass > totalMass) {
				totalMass = mass;
				total = peak;
			}
		}
		int totalP = total.x;
		int totalQ = total.y;
		int n = totalP + totalQ;
		int m = n / 2;
		int[][] points = new int[m + 1][m + 1];
		for (Point peak : peaks) {
			int p = peak.x;
			int q = peak.y;
			if (p > totalP || q > totalQ || p + q == n) {
				continue;
			}
			if (p + q > m) {
				p = totalP - p;
				q = totalQ - q;
			}
			points[p][q]++;
		}
		int[][][] a = new int[m + 1][m + 1][m + 1];
		int[][][] previ = new int[m + 1][m + 1][m + 1];
		int[][][] prevj = new int[m + 1][m + 1][m + 1];
		a[0][0][0] = 1;
		for (int k = 0; k < m; k++) {
			for (int i = 0; i <= k; i++) {
				for (int j = 0; j <= k; j++) {
					for (int ii = i; ii <= i + 1; ii++) {
						for (int jj = j; jj <= j + 1; jj++) {
							int pts = a[k][i][j];
							pts += points[ii][k + 1 - ii];
							if (ii != jj) {
								pts += points[jj][k + 1 - jj];
							}
							if (pts > a[k + 1][ii][jj]) {
								a[k + 1][ii][jj] = pts;
								previ[k + 1][ii][jj] = i;
								prevj[k + 1][ii][jj] = j;
							}
						}
					}
				}
			}
		}
		int mid = n % 2;
		int best = -1;
		int besti = 0;
		int bestj = 0;
		for (int i = 0; i <= m; i++) {
			for (int mp = 0; mp <= mid; mp++) {
				int j = totalP - i - mp;
				if (j < 0 || j > m) {
					continue;
				}
				if (a[m][i][j] > best) {
					best = a[m][i][j];
					besti = i;
					bestj = j;
				}
			}
		}
		int i = besti;
		int j = bestj;
		String s = "";
		if (mid == 1) {
			s = (i + j == totalP) ? "Q" : "P";
		}
		for (int k = m; k > 0; k--) {
			int ii = previ[k][i][j];
			int jj = prevj[k][i][j];
			s = ((i == ii) ? "Q" : "P") + s + ((j == jj) ? "Q" : "P");
			i = ii;
			j = jj;
		}
		out.println(s);
	}

	public static void main(String[] args) throws IOException {
		Locale.setDefault(Locale.US);
		in = new BufferedReader(new FileReader(inputFileName));
		out = new PrintWriter(outputFileName);
		new identification_md().run();
		in.close();
		out.close();
	}
}

