/******************************************************************/
/* ACM ICPC 2012-2013                                             */
/* Northeastern European Regional Contest                         */
/* St Petersburg - Barnaul - Tashkent - Tbilisi, December 2, 2012 */
/******************************************************************/
/* Problem K. Kingdom Reunion                                     */
/*                                                                */
/* Original idea         Maxim Buzdalov                           */
/* Problem statement     Maxim Buzdalov                           */
/* Test set              Maxim Buzdalov                           */
/******************************************************************/
/* Solution                                                       */
/*                                                                */
/* Author                Georgiy Korneev                          */
/******************************************************************/

import java.io.*;
import java.util.*;

class kingdom_gk {
    static Scanner in;
    static PrintWriter out;

    static double edgeY;
    static double dEdgeY;
    static final double EPS = 1e-4;
    static final Comparator<Edge> EDGE_COMPARATOR = new Comparator<Edge>() {
        public int compare(Edge e1, Edge e2) {
            for (double d : new double[]{e1.x(edgeY) - e2.x(edgeY), dEdgeY * (e1.d() - e2.d())}) {
                if (Math.abs(d) > EPS / 100) {
                    return d < 0 ? -1 : 1;
                }
            }
            return 0;
        }
    };

    static class Point {
        double x, y;

        Point(double x, double y) {
            this.x = x;
            this.y = y;
        }

    }

    static boolean zero(double a) {
        return Math.abs(a) < EPS;
    }

    static class Edge implements Comparable<Edge> {
        Point p;
        Edge next, prev, twin;

        Edge(Point p) {
            this.p = p;
            next = prev = this;
        }

        public int compareTo(Edge e) {
            return !zero(p.y - e.p.y) ? Double.compare(p.y, e.p.y) : Double.compare(p.x, e.p.x);
        }

        Edge append(Point p) {
            Edge that = new Edge(p);
            Edge t = prev.next; prev.next = that.next; that.next = t;
            prev.next.prev = prev;
            that.next.prev = that;
            return prev;
        }

        List<Edge> toList() {
            ArrayList<Edge> list = new ArrayList<>();
            Edge e = this;
            do {
                list.add(e);
                e = e.next;
            } while (e != this);
            return list;
        }

        double x(double y) {
            return p.x + (y - p.y) * d();
        }

        double d() {
            return (next.p.x - p.x) / (next.p.y - p.y);
        }

        boolean eq(Edge e) {
            return this.compareTo(e) == 0;
        }

        double area() {
            double area = 0;
            for (Edge edge : toList()) {
                area += (edge.next.p.x - edge.p.x) * (edge.next.p.y + edge.p.y);
            }
            return area;
        }
    }

    void run() throws Exception {
        Edge a = readPoly();
        Edge b = readPoly();
        Edge c = readPoly();

        List<Edge> edges = a.toList();
        edges.addAll(b.toList());
        edges.addAll(c.toList());

        NavigableSet<Edge> points = new TreeSet<>();
        for (Edge edge : edges) {
            points.add(new Edge(edge.p).append(new Point(edge.p.x, edge.p.y + 1)).prev);
        }

        if (!isPoly(a, points)) {
            out.println("Aastria is not a polygon");
        } else if (!isPoly(b, points)) {
            out.println("Abstria is not a polygon");
        } else if (!isPoly(c, points)) {
            out.println("Aabstria is not a polygon");
        } else if (polyIntersects(a, b)){
            out.println("Aastria and Abstria intersect");
        } else if (notCovers(a, c) || notCovers(b, c)){
            out.println("The union of Aastria and Abstria is not equal to Aabstria");
        } else {
            out.println("OK");
        }
    }

    boolean notCovers(Edge a, Edge c) {
        NavigableSet<Edge> edgesC = new TreeSet<>(c.toList());
        for (Edge edge : a.toList()) {
            Edge edgeC = edgesC.floor(edge);
            if (edgeC != null && edgeC.compareTo(edge) == 0) {
                if (edgeC.next.eq(edge.next) && edge.twin != null) {
                    return true;
                }
            } else if (edge.twin == null) {
                return true;
            }
        }
        return false;
    }

    double sign(double v) {
        return Math.abs(v) < EPS ? 0 : v < 0 ? -1 : +1;
    }

    boolean intersects(Edge e1, Edge e2) {
        if (e1 == null || e2 == null || e1.eq(e2) || e1.next.eq(e2) || e1.eq(e2.next) || e1.next.eq(e2.next)) {
            return false;
        }
        double v1 = sign(v(e2.p, e2.next.p, e1.p));
        double v2 = sign(v(e2.p, e2.next.p, e1.next.p));
        double v3 = sign(v(e1.p, e1.next.p, e2.p));
        double v4 = sign(v(e1.p, e1.next.p, e2.next.p));

        return (!zero(v1) || !zero(v2) || !zero(v3) || !zero(v4)) && v1 * v2 < EPS && v3 * v4 < EPS;
    }

    boolean isPoly(Edge a, NavigableSet<Edge> points) {
        if (a.toList().size() < 3 || a.area() < EPS) {
            return false;
        }

        List<Edge> events = a.toList();
        events.addAll(points);
        Collections.sort(events);

        IdentityHashMap<Edge, NavigableSet<Edge>> subdivisions = new IdentityHashMap<>();
        for (Edge edge : events) {
            subdivisions.put(edge, new TreeSet<Edge>());
        }

        List<Edge> add = new ArrayList<>();
        List<Edge> remove = new ArrayList<>();
        NavigableSet<Edge> edges = new TreeSet<>(EDGE_COMPARATOR);
        for (Edge edge : events) {
            edgeY = edge.p.y;
            dEdgeY = 0;

            if (edge.next.next == edge) {
                Edge divided = edges.floor(edge);
                if (divided != null && zero(v(divided.p, divided.next.p, edge.p))) {
                    subdivisions.get(divided).add(edge);
                }
            } else {
                (edge.compareTo(edge.prev) < 0 ? add : remove).add(edge.prev);
                (edge.compareTo(edge.next) < 0 ? add : remove).add(edge);
                if (addRemove(edges, add, remove)) {
                    return false;
                }
            }
        }

        for (Map.Entry<Edge, NavigableSet<Edge>> entry : subdivisions.entrySet()) {
            Edge edge = entry.getKey();
            NavigableSet<Edge> ps = entry.getValue();
            if (edge.compareTo(edge.next) > 0) {
                ps = ps.descendingSet();
            }
            for (Edge point : ps) {
                if (!edge.eq(point) && !edge.next.eq(point)) {
                    edge = edge.next.append(point.p);
                }
            }
        }
        return true;
    }

    boolean polyIntersects(Edge a, Edge b) {
        NavigableSet<Edge> edgesA = new TreeSet<>(a.toList());
        for (Edge edgeB : b.toList()) {
            Edge edgeA = edgesA.floor(edgeB);
            if (edgeA == null || edgeA.compareTo(edgeB) != 0) {
                continue;
            }
            if (
                v(edgeA.p, edgeA.next.p, edgeB.next.p) < -EPS && v(edgeA.p, edgeA.prev.p, edgeB.next.p) > EPS ||
                v(edgeA.p, edgeA.next.p, edgeB.prev.p) < -EPS && v(edgeA.p, edgeA.prev.p, edgeB.prev.p) > EPS ||
                edgeA.next.eq(edgeB.next) || edgeA.prev.eq(edgeB.prev)
            ) {
                return true;
            }
            if (edgeA.next.eq(edgeB.prev)) {
                edgeA.twin = edgeB.prev;
                edgeB.prev.twin = edgeA;
            }
        }

        List<Edge> events = new ArrayList<>();
        events.add(new Edge(new Point(Integer.MAX_VALUE, Integer.MAX_VALUE)));
        events.addAll(a.toList());
        events.addAll(b.toList());
        Collections.sort(events);

        NavigableSet<Edge> edges = new TreeSet<>(EDGE_COMPARATOR);
        List<Edge> remove = new ArrayList<>();
        List<Edge> add = new ArrayList<>();
        Edge prev = new Edge(new Point(Integer.MIN_VALUE, Integer.MIN_VALUE));
        for (Edge edge : events) {
            if (!prev.eq(edge)) {
                edgeY = prev.p.y;
                if (addRemove(edges, add, remove)) return true;
                prev = edge;
            }
            (edge.compareTo(edge.prev) < 0 ? add : remove).add(edge.prev);
            (edge.compareTo(edge.next) < 0 ? add : remove).add(edge);
        }

        return diff(b, a) || diff(a, b);
    }

    private boolean addRemove(NavigableSet<Edge> edges, List<Edge> add, List<Edge> remove) {
        dEdgeY = -1;
        for (Edge e : remove) {
            if (e.twin != null && e.compareTo(e.twin) < 0) {
                continue;
            }
            edges.remove(e);
            if (intersects(edges.ceiling(e), edges.floor(e))) {
                return true;
            }
        }
        remove.clear();

        dEdgeY = 1;
        for (Edge e : add) {
            if (e.twin != null && e.compareTo(e.twin) < 0) {
                continue;
            }
            if (!edges.add(e) || intersects(e, edges.higher(e)) || intersects(e, edges.lower(e))) {
                return true;
            }
        }
        add.clear();
        return false;
    }

    private boolean diff(Edge pa, Edge pb) {
        NavigableSet<Edge> edgesB = new TreeSet<>(pb.toList());
        for (Edge a : pa.toList()) {
            Edge b = edgesB.floor(a);
            if (b == null || b.compareTo(a) != 0) {
                int c = 0;
                for (Edge e : edgesB) {
                    if (e.x(a.p.y) > a.p.x && between(e.p.y, a.p.y, e.next.p.y)) {
                        c++;
                    }
                }
                return c % 2 == 1;
            }
        }
        return false;
    }

    boolean between(double a, double v, double b) {
        return a - v < EPS && v < b - EPS || b - v < EPS && v < a - EPS;
    }

    Edge readPoly() {
        int n = in.nextInt();
        List<Point> points = new ArrayList<>(n);
        double cos = 0.9072348234;
        double sin = Math.sqrt(1 - cos * cos);
        for (int i = 0; i < n; i++) {
            int x = in.nextInt();
            int y = in.nextInt();
            points.add(new Point(x * cos + y * sin, y * cos - x * sin));
        }
        Edge polygon = createPolygon(points);
        if (polygon.area() >= 0) {
            return polygon;
        }
        Collections.reverse(points);
        return createPolygon(points);
    }

    Edge createPolygon(List<Point> points) {
        Edge root = new Edge(points.get(0));
        for (int i = 1; i < points.size(); i++) {
            root.append(points.get(i));
        }
        return root;
    }

    double v(Point p0, Point p1, Point p2) {
        return (p1.x - p0.x) * (p2.y - p0.y) - (p1.y - p0.y) * (p2.x - p0.x);
    }

    static class Scanner {
        final BufferedReader reader;
        StringTokenizer tokenizer = new StringTokenizer("");

        Scanner(File file) throws FileNotFoundException {
            reader = new BufferedReader(new FileReader(file));
        }

        int nextInt() {
            return Integer.parseInt(next());
        }

        String next() {
            while (!tokenizer.hasMoreTokens()) {
                try {
                    tokenizer = new StringTokenizer(reader.readLine());
                } catch (IOException e) {
                    throw new AssertionError();
                }
            }
            return tokenizer.nextToken();
        }

        void close() {
        }
    }

    public static void main(String[] args) throws Exception {
        Locale.setDefault(Locale.ENGLISH);
        in = new Scanner(new File("kingdom.in"));
        out = new PrintWriter("kingdom.out");

        new kingdom_gk().run();

        in.close();
        out.close();
    }
}

