/******************************************************************/
/* ACM ICPC 2012-2013                                             */
/* Northeastern European Regional Contest                         */
/* St Petersburg - Barnaul - Tashkent - Tbilisi, December 2, 2012 */
/******************************************************************/
/* Problem L. Labyrinth of the Minotaur                           */
/*                                                                */
/* Original idea         Georgiy Korneev                          */
/* Problem statement     Georgiy Korneev                          */
/* Test set              Georgiy Korneev                          */
/******************************************************************/
/* Solution                                                       */
/*                                                                */
/* Author                Georgiy Korneev                          */
/******************************************************************/

import java.io.File;
import java.io.PrintWriter;
import java.util.Scanner;

public class labyrinth_gk_n2logn {
    static Scanner in;
    static PrintWriter out;
    int w, h;
    boolean[][] labyrinth;

    class Cells {
        int[][] sums = new int[h + 1][w + 1];

        public Cells(boolean[][] labyrinth) {
            for (int i = 1; i <= labyrinth.length; i++) {
                int sum = 0;
                for (int j = 1; j <= labyrinth[i - 1].length; j++) {
                    sum += (labyrinth[i - 1][j - 1] ? 1 : 0);
                    sums[i][j] = sum + (i > 0 ? sums[i - 1][j] : 0);
                }
            }
        }

        public boolean exist(int s, int i, int j) {
            return sums[i + s][j + s] - sums[i][j + s] - sums[i + s][j] + sums[i][j] > 0;
        }
    }

    static int[] DX = {1, 0, -1, 0};
    static int[] DY = {0, 1, 0, -1};

    void run() {
        w = in.nextInt();
        h = in.nextInt();
        in.nextLine();
        labyrinth = new boolean[h][w];
        for (int i =  0; i < h; i++) {
            char[] line = in.nextLine().toCharArray();
            for (int j = 0; j < w; j++) {
                labyrinth[i][j] = line[j] == '#';
            }
        }

        Cells path1 = new Cells(findWay(0, 1));
        Cells path2 = new Cells(findWay(1, -1));

        labyrinth[0][0] = true;
        labyrinth[h - 1][w - 1] = true;
        Cells walls = new Cells(labyrinth);

        int bestS = Integer.MAX_VALUE;
        int bestI = -1;
        int bestJ = -1;
        for (int i = 0; i < h; i++) {
            for (int j = 0; j < w; j++) {
                int l = 0;
                int r = Math.min(h - i, w - j);
                if (path1.exist(r, i, j) && path2.exist(r, i, j)) {
                    while (l < r - 1) {
                        int m = (l + r) / 2;
                        if (path1.exist(m, i, j) && path2.exist(m, i, j)) {
                            r = m;
                        } else {
                            l = m;
                        }
                    }
                    if (bestS > r && !walls.exist(r, i, j)) {
                        bestS = r;
                        bestI = i;
                        bestJ = j;
                    }
                }
            }
        }
        if (bestS == Integer.MAX_VALUE) {
            out.println("Impossible");
        } else {
            out.println(bestS + " " + (bestJ + 1) + " " + (bestI + 1));
        }
    }

    private boolean[][] findWay(int d, int dd) {
        boolean[][] way = new boolean[h][w];
        int x = 0;
        int y = 0;
        while (x != w - 1 || y != h - 1) {
            way[y][x] = true;
            d = (d - dd + 4) % 4;
            for (int i = 0; i < 3; i++) {
                int nx = x + DX[d];
                int ny = y + DY[d];
                if (0 <= nx && nx < w && 0 <= ny && ny < h && !labyrinth[ny][nx]) {
                    x = nx;
                    y = ny;
                    break;
                }
                d = (d + dd + 4) % 4;
            }
        }
        return way;
    }

    public static void main(String[] args) throws Exception {
        in = new Scanner(new File("labyrinth.in"));
        out = new PrintWriter("labyrinth.out");

        new labyrinth_gk_n2logn().run();

        in.close();
        out.close();
    }
}

